;;; flyspell-flymake.el --- Flymake backend for flyspell, i.e. flyspell sans flyspell  -*- lexical-binding: t; -*-

;; Copyright (C) 2018 Free Software Foundation, Inc.

;; Author: João Távora <joaotavora@gmail.com>
;; Keywords:

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>.

;;; Commentary:
;;

;;; Code:
(require 'cl-lib)
(require 'flyspell)
(require 'flymake)

(defun flyspell--sans-flyspell-region (&optional beg end)
  "Fake-flyspell region between BEG and END.
Intead of making overlays, return Flymake diagnostics"
  (let (diags)
    (cl-letf (((symbol-function 'overlays-in)
               (lambda (beg end)
                 (cl-remove-if-not
                  (lambda (diag)
                    (let ((diag-beg (flymake-diagnostic-beg diag))
                          (diag-end (flymake-diagnostic-end diag)))
                      (and (< diag-beg end) (> diag-end beg))))
                  diags)))
              ((symbol-function 'make-overlay)
               (lambda (beg end &optional buffer &rest ignored)
                 (let ((diag (flymake-make-diagnostic
                              (or buffer (current-buffer))
                              beg end 'flyspell-flymake-mispelling
                              "I thing you misspelled this.")))
                   (push diag diags)
                   diag)))
              ((symbol-function 'delete-overlay)
               (lambda (diag)
                 (setq diags (delq diag diags))))
              ((symbol-function 'overlayp)
               (lambda (thing) (flymake--diag-p thing)))
              ((symbol-function 'overlay-put)
               (lambda (diag prop value) (setf ( flymake--diag-data diag)
                                               (plist-put
                                                (flymake-diagnostic-data diag)
                                                prop value))))
              ((symbol-function 'overlay-get)
               (lambda (diag prop) (plist-get (flymake-diagnostic-data diag)
                                              prop)))
              ((symbol-function 'overlay-properties) #'flymake-diagnostic-data))
      (save-excursion
        (if (and beg end) (flyspell-region beg end)
          (flyspell-region (point-min) (point-max)))))
    diags))

(put 'flyspell-flymake-mispelling 'flymake-severity (warning-numeric-level :warning))
(put 'flyspell-flymake-mispelling 'flymake-category :note)
(put 'flyspell-flymake-mispelling 'flymake-overlay-control
     `((mouse-face . highlight)
       (keymap . ,(let ((map (make-sparse-keymap)))
                    (define-key map [mouse-2] 'flyspell-correct-word)
                    map))))

;;;###autoload
(cl-defun flyspell-flymake (report-fn &key changes-start changes-end
                                      &allow-other-keys)
  (let ((diagnostics (flyspell--sans-flyspell-region changes-start changes-end)))
    (apply report-fn diagnostics
           (if (and changes-start changes-end)
               (list :region (cons changes-start changes-end))))
    diagnostics))
;;;###autoload
(cl-defun flyspell-prog-flymake (report-fn &rest args)
  (let* ((flyspell-generic-check-word-predicate #'flyspell-generic-progmode-verify))
    (apply #'flyspell-flymake report-fn args)))

(provide 'flyspell-flymake)
;;; flyspell-flymake.el ends here


